//-------------------------------------------------------------
// Acclight: experimental environment for 
//           multipass-rendering, lighting, shading and shadows
//-------------------------------------------------------------
//
// Daniel Bielser - january  1999
// Reto Ltolf    - december 1999
//-------------------------------------------------------------


#ifndef _ACCLIGHT_H_
#define _ACCLIGHT_H_


//includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <GL/glut.h>


//constants
const double AL_PI = 3.14159265358979323846;


//defines
#ifndef TRUE
#define TRUE (0==0)
#endif
#ifndef FALSE
#define FALSE (!TRUE)
#endif

#define AL_NBR_JITTER_SAMPLE 16

//define lights:
#define AL_NBR_Y_LIGHTS 5
#define AL_NBR_Z_LIGHTS 3
#define AL_NBR_OF_LIGHTS (AL_NBR_Y_LIGHTS * AL_NBR_Z_LIGHTS) 
#define AL_DELTA_Y_LIGHTS 0.01
#define AL_DELTA_Z_LIGHTS 0.01

//define display lists:
#define AL_TABLE_LIST 1
#define AL_CHAIR_LIST 2

//define errors:
#define AL_FILE_ERROR -1

//define light positions
#define AL_LIGHT_POS_X  0.0
#define AL_LIGHT_POS_Y  2.1
#define AL_LIGHT_POS_Z  4.0

//define object position
#define AL_CENTER_X 2.0
#define AL_CENTER_Y 2.0
#define AL_CENTER_Z 5.0

//define camera position
#define AL_CAM_POS_X  0.0
#define AL_CAM_POS_Y  3.0
#define AL_CAM_POS_Z  0.0

//define wall positions
#define AL_X_WALL_POS    5.0
#define AL_Y_WALL_POS    0.95
#define AL_Z_WALL_POS    7.0
#define AL_WALL_SIZE     7.0
#define AL_WALL_SUB_DIV 30

//define texture options
#define AL_TEXTURE_X_SIZE 512
#define AL_TEXTURE_Y_SIZE 512
#define AL_TEX_SCAL         1.5
#define AL_WALL_TEX_SCAL   10



//function prototypes
int read_texmap(char *acFile, int *width, int *height,
	    GLubyte TexMap[AL_TEXTURE_X_SIZE][AL_TEXTURE_Y_SIZE][3]);
void setLighting(void);
void setMaterial(GLfloat *ambient, GLfloat *diffuse, GLfloat *specular, 
	    GLfloat shininess, GLfloat *emission);
void buildTexturedRectangle(float rectangle_width, float rectangle_height,
		       float rectangle_depth, float sub_div);
void buildTexturedCube(float cube_width, float cube_height, float cube_depth);
void buildTable(void);
void buildChair(void);
void buildLightSource(int nbr_y, int nbr_z, float delta_y, float delta_z);
void drawObjects(void);
void drawTeapot(void);
void drawShadow(GLfloat *projection_matrix, int light_nbr);
void generateShadowProjections(GLfloat *projection_matrixX, GLfloat *projection_matrixY,
			       GLfloat *projection_matrixZ, int light_nbr);
void displayOnePass(int light_nbr);
void init(void);
void accFrustum(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top,
	    GLdouble near_, GLdouble far_, GLdouble pixdx, GLdouble pixdy,
	    GLdouble eyedx, GLdouble eyedy, GLdouble focus);
void accPerspective(GLdouble fovy, GLdouble aspect, GLdouble near_, GLdouble far_,
	       GLdouble pixdx, GLdouble pixdy, GLdouble eyedx, GLdouble eyedy,
	       GLdouble focus);

//callback prototypes
void callback_display(void);
void callback_reshape(int w, int h);



#endif



//function descriptions

//------------------------------------------------------------
// setMaterial
// sets the material properties for the ambient, diffuse
// and specular reflection and for the shininess of an object.
//------------------------------------------------------------
// --> ambient[4]    ambient reflection coefficients
// --> diffuse[4]    diffuse reflection ceofficients
// --> specular[4]   specular reflection coefficients
// --> shininess     shininess of an object
// --> emission[4]   self-emission of an object
//------------------------------------------------------------

//------------------------------------------------------
// setLighting
// defines a light and sets the ligthing properties
//------------------------------------------------------

//-------------------------------------------------------------------
// generateShadowProjections
// generates the projection matrices for the shadow projection
// onto the walls: 
// x = AL_X_WALL_POS, y = AL_Y_WALL_POS and z = AL_Z_WALL_POS
//-------------------------------------------------------------------
// <-> projection_matrixX[16]
//     projection matrix for shadow projection onto the x = AL_X_WALL
// <-> projection_matrixY[16]
//     projection matrix for shadow projection onto the y = AL_Y_WALL
// <-> projection_matrixZ[16]
//     projection matrix for shadow projection onto the z = AL_Z_WALL
// --> light_nbr  index into the light_position - array
//-------------------------------------------------------------------

//-----------------------------------------------------
// drawShadow
// draws the shadows of the objects
//-----------------------------------------------------
// --> projection_matrix[16]
//     projection matrix which describes the shadow
//     projection onto a wall
// --> light_nbr  index into the light_position - array
//-----------------------------------------------------

//--------------------------------------------------------------
// buildLightSource
// defines a spatial rectangular light in the y-z-plane,
// consisting of nbr_y*nbr_z spotlights by setting the global
// array of light positions: light_position.
// The first light has the position
// (AL_LIGHT_POS_X, AL_LIGHT_POS_Y, AL_LIGHT_POS_Z, 1.0).
// All further lights are at position n*delta_y in y-direction
// (0 < n < nbr_y) and m*delta_z in z-direction (0 < m < nbr_z).
//--------------------------------------------------------------
// --> nbr_y	number of lights in the direction of the y-axis
// --> nbr_z	number of lights in the direction of the z-axis
// --> delta_y	distance between two lights in y-direction
// --> delta_z	distance between two lights in z-direction
//
// <-> light_position[nbr_y * nbr_z][4]
//               global array of 4-dimensional light positions
//               (global definition, therefore not a parameter)
//--------------------------------------------------------------
